@@ -17,69 +17,70 @@
 # LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 # THE SOFTWARE.
+"""Generic utility helpers."""
 
-import glob
-import imp
+import contextlib
+import inspect
+import logging
 import os
-
-import six
-from ansible import constants
-from ansible.errors import AnsibleError
-
-try:
-    # Try to import the Ansible 2 module first, it's the future-proof one
-    from ansible.parsing.splitter import split_args
-
-except ImportError:
-    # Fallback on the Ansible 1.9 module
-    from ansible.module_utils.splitter import split_args
+import pprint
+import subprocess
+from argparse import Namespace
+from collections import OrderedDict
+from functools import lru_cache
+from pathlib import Path
+from typing import Callable, ItemsView, List, Optional, Tuple
 
 import yaml
+from ansible import constants
+from ansible.errors import AnsibleError, AnsibleParserError
+from ansible.parsing.dataloader import DataLoader
+from ansible.parsing.mod_args import ModuleArgsParser
+from ansible.parsing.splitter import split_args
+from ansible.parsing.yaml.constructor import AnsibleConstructor
+from ansible.parsing.yaml.loader import AnsibleLoader
+from ansible.parsing.yaml.objects import AnsibleSequence
+from ansible.plugins.loader import module_loader
+from ansible.template import Templar
 from yaml.composer import Composer
-from yaml.constructor import Constructor
-import ruamel.yaml
+from yaml.representer import RepresenterError
 
-try:
-    from ansible.utils import parse_yaml_from_file
-    from ansible.utils import path_dwim
-    from ansible.utils.template import template as ansible_template
-    from ansible.utils import module_finder
-    module_loader = module_finder
-    ANSIBLE_VERSION = 1
-except ImportError:
-    from ansible.parsing.dataloader import DataLoader
-    from ansible.template import Templar
-    from ansible.parsing.mod_args import ModuleArgsParser
-    from ansible.parsing.yaml.constructor import AnsibleConstructor
-    from ansible.parsing.yaml.loader import AnsibleLoader
-    from ansible.errors import AnsibleParserError
-    ANSIBLE_VERSION = 2
+from ansiblelint.constants import (
+    ANSIBLE_FAILURE_RC, CUSTOM_RULESDIR_ENVVAR, DEFAULT_RULESDIR, FileType,
+)
+from ansiblelint.errors import MatchError
+from ansiblelint.file_utils import normpath
 
-    # ansible-lint doesn't need/want to know about encrypted secrets, but it needs
-    # Ansible 2.3+ allows encrypted secrets within yaml files, so we pass a string
-    # as the password to enable such yaml files to be opened and parsed successfully.
-    DEFAULT_VAULT_PASSWORD = 'x'
+# ansible-lint doesn't need/want to know about encrypted secrets, so we pass a
+# string as the password to enable such yaml files to be opened and parsed
+# successfully.
+DEFAULT_VAULT_PASSWORD = 'x'
 
-    def parse_yaml_from_file(filepath):
-        dl = DataLoader()
-        if hasattr(dl, 'set_vault_password'):
-            dl.set_vault_password(DEFAULT_VAULT_PASSWORD)
-        return dl.load_from_file(filepath)
+PLAYBOOK_DIR = os.environ.get('ANSIBLE_PLAYBOOK_DIR', None)
 
-    def path_dwim(basedir, given):
-        dl = DataLoader()
-        dl.set_basedir(basedir)
-        return dl.path_dwim(given)
 
-    def ansible_template(basedir, varname, templatevars, **kwargs):
-        dl = DataLoader()
-        dl.set_basedir(basedir)
-        templar = Templar(dl, variables=templatevars)
-        return templar.template(varname, **kwargs)
-    try:
-        from ansible.plugins import module_loader
-    except ImportError:
-        from ansible.plugins.loader import module_loader
+_logger = logging.getLogger(__name__)
+
+
+def parse_yaml_from_file(filepath: str) -> dict:
+    dl = DataLoader()
+    if hasattr(dl, 'set_vault_password'):
+        dl.set_vault_password(DEFAULT_VAULT_PASSWORD)
+    return dl.load_from_file(filepath)
+
+
+def path_dwim(basedir: str, given: str) -> str:
+    dl = DataLoader()
+    dl.set_basedir(basedir)
+    return dl.path_dwim(given)
+
+
+def ansible_template(basedir, varname, templatevars, **kwargs):
+    dl = DataLoader()
+    dl.set_basedir(basedir)
+    templar = Templar(dl, variables=templatevars)
+    return templar.template(varname, **kwargs)
+
 
 LINE_NUMBER_KEY = '__line__'
 FILENAME_KEY = '__file__'
@@ -106,24 +107,6 @@ BLOCK_NAME_TO_ACTION_TYPE_MAP = {
 }
 
 
-def load_plugins(directory):
-    result = []
-    fh = None
-
-    for pluginfile in glob.glob(os.path.join(directory, '[A-Za-z]*.py')):
-
-        pluginname = os.path.basename(pluginfile.replace('.py', ''))
-        try:
-            fh, filename, desc = imp.find_module(pluginname, [directory])
-            mod = imp.load_module(pluginname, fh, filename, desc)
-            obj = getattr(mod, pluginname)()
-            result.append(obj)
-        finally:
-            if fh:
-                fh.close()
-    return result
-
-
 def tokenize(line):
     tokens = line.lstrip().split(" ")
     if tokens[0] == '-':
@@ -145,7 +128,7 @@ def tokenize(line):
     return (command, args, kwargs)
 
 
-def _playbook_items(pb_data):
+def _playbook_items(pb_data: dict) -> ItemsView:
     if isinstance(pb_data, dict):
         return pb_data.items()
     elif not pb_data:
@@ -154,7 +137,17 @@ def _playbook_items(pb_data):
         return [item for play in pb_data for item in play.items()]
 
 
-def find_children(playbook, playbook_dir):
+def _rebind_match_filename(filename: str, func) -> Callable:
+    def func_wrapper(*args, **kwargs):
+        try:
+            return func(*args, **kwargs)
+        except MatchError as e:
+            e.filename = filename
+            raise e
+    return func_wrapper
+
+
+def find_children(playbook: Tuple[str, str], playbook_dir: str) -> List:
     if not os.path.exists(playbook[0]):
         return []
     if playbook[1] == 'role':
@@ -168,7 +161,8 @@ def find_children(playbook, playbook_dir):
     basedir = os.path.dirname(playbook[0])
     items = _playbook_items(playbook_ds)
     for item in items:
-        for child in play_children(basedir, item, playbook[1], playbook_dir):
+        for child in _rebind_match_filename(playbook[0], play_children)(
+                basedir, item, playbook[1], playbook_dir):
             if "$" in child['path'] or "{{" in child['path']:
                 continue
             valid_tokens = list()
@@ -190,7 +184,7 @@ def template(basedir, value, vars, fail_on_undefined=False, **kwargs):
                                  **dict(kwargs, fail_on_undefined=fail_on_undefined))
         # Hack to skip the following exception when using to_json filter on a variable.
         # I guess the filter doesn't like empty vars...
-    except (AnsibleError, ValueError):
+    except (AnsibleError, ValueError, RepresenterError):
         # templating failed, so just keep value as is.
         pass
     return value
@@ -218,7 +212,7 @@ def play_children(basedir, item, parent_type, playbook_dir):
         if v:
             v = template(os.path.abspath(basedir),
                          v,
-                         dict(playbook_dir=os.path.abspath(basedir)),
+                         dict(playbook_dir=PLAYBOOK_DIR or os.path.abspath(basedir)),
                          fail_on_undefined=False)
             return delegate_map[k](basedir, k, v, parent_type)
     return []
@@ -229,53 +223,73 @@ def _include_children(basedir, k, v, parent_type):
     (command, args, kwargs) = tokenize("{0}: {1}".format(k, v))
 
     result = path_dwim(basedir, args[0])
-    if not os.path.exists(result) and not basedir.endswith('tasks'):
-        result = path_dwim(os.path.join(basedir, '..', 'tasks'), v)
+    if not os.path.exists(result):
+        result = path_dwim(os.path.join(os.path.dirname(basedir)), v)
     return [{'path': result, 'type': parent_type}]
 
 
-def _taskshandlers_children(basedir, k, v, parent_type):
+def _taskshandlers_children(basedir, k, v, parent_type: FileType) -> List:
     results = []
     for th in v:
-        if 'include' in th:
-            append_children(th['include'], basedir, k, parent_type, results)
-        elif 'include_tasks' in th:
-            append_children(th['include_tasks'], basedir, k, parent_type, results)
-        elif 'import_playbook' in th:
-            append_children(th['import_playbook'], basedir, k, parent_type, results)
-        elif 'import_tasks' in th:
-            append_children(th['import_tasks'], basedir, k, parent_type, results)
-        elif 'import_role' in th:
-            results.extend(_roles_children(basedir, k, [th['import_role'].get('name')], parent_type,
-                                           main=th['import_role'].get('tasks_from', 'main')))
-        elif 'include_role' in th:
-            results.extend(_roles_children(basedir, k, [th['include_role'].get('name')],
+        with contextlib.suppress(LookupError):
+            children = _get_task_handler_children_for_tasks_or_playbooks(
+                th, basedir, k, parent_type,
+            )
+            results.append(children)
+            continue
+
+        if 'include_role' in th or 'import_role' in th:  # lgtm [py/unreachable-statement]
+            th = normalize_task_v2(th)
+            _validate_task_handler_action_for_role(th['action'])
+            results.extend(_roles_children(basedir, k, [th['action'].get("name")],
                                            parent_type,
-                                           main=th['include_role'].get('tasks_from', 'main')))
-        elif 'block' in th:
-            results.extend(_taskshandlers_children(basedir, k, th['block'], parent_type))
-            if 'rescue' in th:
-                results.extend(_taskshandlers_children(basedir, k, th['rescue'], parent_type))
-            if 'always' in th:
-                results.extend(_taskshandlers_children(basedir, k, th['always'], parent_type))
+                                           main=th['action'].get('tasks_from', 'main')))
+            continue
+
+        if 'block' not in th:
+            continue
+
+        results.extend(_taskshandlers_children(basedir, k, th['block'], parent_type))
+        if 'rescue' in th:
+            results.extend(_taskshandlers_children(basedir, k, th['rescue'], parent_type))
+        if 'always' in th:
+            results.extend(_taskshandlers_children(basedir, k, th['always'], parent_type))
+
     return results
 
 
-def append_children(taskhandler, basedir, k, parent_type, results):
-    # when taskshandlers_children is called for playbooks, the
-    # actual type of the included tasks is the section containing the
-    # include, e.g. tasks, pre_tasks, or handlers.
-    if parent_type == 'playbook':
-        playbook_section = k
-    else:
-        playbook_section = parent_type
-    results.append({
-        'path': path_dwim(basedir, taskhandler),
-        'type': playbook_section
-    })
+def _get_task_handler_children_for_tasks_or_playbooks(
+        task_handler, basedir: str, k, parent_type: FileType,
+) -> dict:
+    """Try to get children of taskhandler for include/import tasks/playbooks."""
+    child_type = k if parent_type == 'playbook' else parent_type
+    task_include_keys = 'include', 'include_tasks', 'import_playbook', 'import_tasks'
+    for task_handler_key in task_include_keys:
+        with contextlib.suppress(KeyError):
+            return {
+                'path': path_dwim(basedir, task_handler[task_handler_key]),
+                'type': child_type,
+            }
+
+    raise LookupError(
+        f'The node contains none of: {", ".join(task_include_keys)}',
+    )
 
 
-def _roles_children(basedir, k, v, parent_type, main='main'):
+def _validate_task_handler_action_for_role(th_action: dict) -> None:
+    """Verify that the task handler action is valid for role include."""
+    module = th_action['__ansible_module__']
+
+    if 'name' not in th_action:
+        raise MatchError(f"Failed to find required 'name' key in {module!s}")
+
+    if not isinstance(th_action['name'], str):
+        raise RuntimeError(
+            f"Value assigned to 'name' key on '{module!s}' is not a string.",
+        )
+
+
+def _roles_children(basedir: str, k, v, parent_type: FileType, main='main') -> list:
     results = []
     for role in v:
         if isinstance(role, dict):
@@ -292,12 +306,12 @@ def _roles_children(basedir, k, v, parent_type, main='main'):
     return results
 
 
-def _load_library_if_exists(path):
+def _load_library_if_exists(path: str) -> None:
     if os.path.exists(path):
         module_loader.add_directory(path)
 
 
-def _rolepath(basedir, role):
+def _rolepath(basedir: str, role: str) -> Optional[str]:
     role_path = None
 
     possible_paths = [
@@ -313,7 +327,7 @@ def _rolepath(basedir, role):
 
     if constants.DEFAULT_ROLES_PATH:
         search_locations = constants.DEFAULT_ROLES_PATH
-        if isinstance(search_locations, six.string_types):
+        if isinstance(search_locations, str):
             search_locations = search_locations.split(os.pathsep)
         for loc in search_locations:
             loc = os.path.expanduser(loc)
@@ -332,7 +346,7 @@ def _rolepath(basedir, role):
     return role_path
 
 
-def _look_for_role_files(basedir, role, main='main'):
+def _look_for_role_files(basedir: str, role: str, main='main') -> list:
     role_path = _rolepath(basedir, role)
     if not role_path:
         return []
@@ -355,42 +369,66 @@ def rolename(filepath):
     idx = filepath.find('roles/')
     if idx < 0:
         return ''
-    role = filepath[idx+6:]
+    role = filepath[idx + 6:]
     role = role[:role.find('/')]
     return role
 
 
 def _kv_to_dict(v):
     (command, args, kwargs) = tokenize(v)
-    return (dict(__ansible_module__=command, __ansible_arguments__=args, **kwargs))
+    return dict(__ansible_module__=command, __ansible_arguments__=args, **kwargs)
 
 
-def normalize_task_v2(task):
-    '''Ensures tasks have an action key and strings are converted to python objects'''
+def _sanitize_task(task: dict) -> dict:
+    """Return a stripped-off task structure compatible with new Ansible.
 
+    This helper takes a copy of the incoming task and drops
+    any internally used keys from it.
+    """
+    result = task.copy()
+    # task is an AnsibleMapping which inherits from OrderedDict, so we need
+    # to use `del` to remove unwanted keys.
+    for k in ['skipped_rules', FILENAME_KEY, LINE_NUMBER_KEY]:
+        if k in result:
+            del result[k]
+    return result
+
+
+# FIXME: drop noqa once this function is made simpler
+# Ref: https://github.com/ansible/ansible-lint/issues/744
+def normalize_task_v2(task: dict) -> dict:  # noqa: C901
+    """Ensure tasks have an action key and strings are converted to python objects."""
     result = dict()
-    mod_arg_parser = ModuleArgsParser(task)
+    if 'always_run' in task:
+        # FIXME(ssbarnea): Delayed import to avoid circular import
+        # See https://github.com/ansible/ansible-lint/issues/880
+        # noqa: # pylint:disable=cyclic-import,import-outside-toplevel
+        from ansiblelint.rules.AlwaysRunRule import AlwaysRunRule
+
+        raise MatchError(
+            rule=AlwaysRunRule,
+            filename=task[FILENAME_KEY],
+            linenumber=task[LINE_NUMBER_KEY])
+
+    sanitized_task = _sanitize_task(task)
+    mod_arg_parser = ModuleArgsParser(sanitized_task)
     try:
         action, arguments, result['delegate_to'] = mod_arg_parser.parse()
     except AnsibleParserError as e:
         try:
             task_info = "%s:%s" % (task[FILENAME_KEY], task[LINE_NUMBER_KEY])
-            del task[FILENAME_KEY]
-            del task[LINE_NUMBER_KEY]
         except KeyError:
             task_info = "Unknown"
-        try:
-            import pprint
-            pp = pprint.PrettyPrinter(indent=2)
-            task_pprint = pp.pformat(task)
-        except ImportError:
-            task_pprint = task
-        raise SystemExit("Couldn't parse task at %s (%s)\n%s" % (task_info, e.message, task_pprint))
+        pp = pprint.PrettyPrinter(indent=2)
+        task_pprint = pp.pformat(sanitized_task)
+
+        _logger.critical("Couldn't parse task at %s (%s)\n%s", task_info, e.message, task_pprint)
+        raise SystemExit(ANSIBLE_FAILURE_RC)
 
     # denormalize shell -> command conversion
     if '_uses_shell' in arguments:
         action = 'shell'
-        del(arguments['_uses_shell'])
+        del arguments['_uses_shell']
 
     for (k, v) in list(task.items()):
         if k in ('action', 'local_action', 'args', 'delegate_to') or k == action:
@@ -404,19 +442,21 @@ def normalize_task_v2(task):
 
     if '_raw_params' in arguments:
         result['action']['__ansible_arguments__'] = arguments['_raw_params'].split(' ')
-        del(arguments['_raw_params'])
+        del arguments['_raw_params']
     else:
         result['action']['__ansible_arguments__'] = list()
 
     if 'argv' in arguments and not result['action']['__ansible_arguments__']:
         result['action']['__ansible_arguments__'] = arguments['argv']
-        del(arguments['argv'])
+        del arguments['argv']
 
     result['action'].update(arguments)
     return result
 
 
-def normalize_task_v1(task):
+# FIXME: drop noqa once this function is made simpler
+# Ref: https://github.com/ansible/ansible-lint/issues/744
+def normalize_task_v1(task):  # noqa: C901
     result = dict()
     for (k, v) in task.items():
         if k in VALID_KEYS or k.startswith('with_'):
@@ -428,7 +468,7 @@ def normalize_task_v1(task):
             else:
                 result[k] = v
         else:
-            if isinstance(v, six.string_types):
+            if isinstance(v, str):
                 v = _kv_to_dict(k + ' ' + v)
             elif not v:
                 v = dict(__ansible_module__=k)
@@ -457,21 +497,18 @@ def normalize_task_v1(task):
         #   module: ec2
         #   etc...
         result['action']['__ansible_module__'] = result['action']['module']
-        del(result['action']['module'])
+        del result['action']['module']
     if 'args' in result:
         result['action'].update(result.get('args'))
-        del(result['args'])
+        del result['args']
     return result
 
 
 def normalize_task(task, filename):
     ansible_action_type = task.get('__ansible_action_type__', 'task')
     if '__ansible_action_type__' in task:
-        del(task['__ansible_action_type__'])
-    if ANSIBLE_VERSION < 2:
-        task = normalize_task_v1(task)
-    else:
-        task = normalize_task_v2(task)
+        del task['__ansible_action_type__']
+    task = normalize_task_v2(task)
     task[FILENAME_KEY] = filename
     task['__ansible_action_type__'] = ansible_action_type
     return task
@@ -543,12 +580,12 @@ def get_normalized_tasks(yaml, file):
     return res
 
 
+@lru_cache(maxsize=128)
 def parse_yaml_linenumbers(data, filename):
-    """Parses yaml as ansible.utils.parse_yaml but with linenumbers.
+    """Parse yaml as ansible.utils.parse_yaml but with linenumbers.
 
     The line numbers are stored in each node's LINE_NUMBER_KEY key.
     """
-
     def compose_node(parent, index):
         # the line number where the previous token has ended (plus empty lines)
         line = loader.line
@@ -557,10 +594,7 @@ def parse_yaml_linenumbers(data, filename):
         return node
 
     def construct_mapping(node, deep=False):
-        if ANSIBLE_VERSION < 2:
-            mapping = Constructor.construct_mapping(loader, node, deep=deep)
-        else:
-            mapping = AnsibleConstructor.construct_mapping(loader, node, deep=deep)
+        mapping = AnsibleConstructor.construct_mapping(loader, node, deep=deep)
         if hasattr(node, '__line__'):
             mapping[LINE_NUMBER_KEY] = node.__line__
         else:
@@ -569,14 +603,10 @@ def parse_yaml_linenumbers(data, filename):
         return mapping
 
     try:
-        if ANSIBLE_VERSION < 2:
-            loader = yaml.Loader(data)
-        else:
-            import inspect
-            kwargs = {}
-            if 'vault_password' in inspect.getargspec(AnsibleLoader.__init__).args:
-                kwargs['vault_password'] = DEFAULT_VAULT_PASSWORD
-            loader = AnsibleLoader(data, **kwargs)
+        kwargs = {}
+        if 'vault_password' in inspect.getfullargspec(AnsibleLoader.__init__).args:
+            kwargs['vault_password'] = DEFAULT_VAULT_PASSWORD
+        loader = AnsibleLoader(data, **kwargs)
         loader.compose_node = compose_node
         loader.construct_mapping = construct_mapping
         data = loader.get_single_data()
@@ -596,72 +626,179 @@ def get_first_cmd_arg(task):
     return first_cmd_arg
 
 
-def append_skipped_rules(pyyaml_data, file_text, file_type):
-    """ Uses ruamel.yaml to parse comments then adds a
-        skipped_rules list to the task (or meta yaml block)
+def is_playbook(filename: str) -> bool:
     """
-    yaml = ruamel.yaml.YAML()
-    ruamel_data = yaml.load(file_text)
+    Check if the file is a playbook.
 
-    if file_type in ('tasks', 'handlers'):
-        ruamel_tasks = ruamel_data
-        pyyaml_tasks = pyyaml_data
-    elif file_type == 'playbook':
-        try:
-            ruamel_tasks = []
-            pyyaml_tasks = []
-            for ruamel_play, pyyaml_play in zip(ruamel_data, pyyaml_data):
-                ruamel_tasks.extend(ruamel_play.get('tasks'))
-                pyyaml_tasks.extend(pyyaml_play.get('tasks'))
-        except (AttributeError, TypeError):
-            return pyyaml_data
-    elif file_type == 'meta':
-        if not isinstance(pyyaml_data, list):
-            return pyyaml_data
-        ruamel_tasks = [ruamel_data]
-        pyyaml_tasks = pyyaml_data
+    Given a filename, it should return true if it looks like a playbook. The
+    function is not supposed to raise exceptions.
+    """
+    # we assume is a playbook if we loaded a sequence of dictionaries where
+    # at least one of these keys is present:
+    playbooks_keys = {
+        "gather_facts",
+        "hosts",
+        "import_playbook",
+        "post_tasks",
+        "pre_tasks",
+        "roles"
+        "tasks",
+    }
+
+    # makes it work with Path objects by converting them to strings
+    if not isinstance(filename, str):
+        filename = str(filename)
+
+    try:
+        f = parse_yaml_from_file(filename)
+    except Exception as e:
+        _logger.warning(
+            "Failed to load %s with %s, assuming is not a playbook.",
+            filename, e)
     else:
-        return pyyaml_data
-
-    if len(ruamel_tasks) != len(pyyaml_tasks):
-        return pyyaml_data
-
-    for ruamel_task, pyyaml_task in zip(ruamel_tasks, pyyaml_tasks):
-        skipped_rules = _get_rule_skips_from_task(ruamel_task)
-        if skipped_rules:
-            pyyaml_task['skipped_rules'] = skipped_rules
-
-    return pyyaml_data
+        if (
+            isinstance(f, AnsibleSequence) and
+            hasattr(f, 'keys') and
+            playbooks_keys.intersection(next(iter(f), {}).keys())
+        ):
+            return True
+    return False
 
 
-def _get_rule_skips_from_task(task):
-    def traverse_yaml(obj):
-        task_comment_obj_strs.append(str(obj.ca.items))
-        if isinstance(obj, dict):
-            for key, val in obj.items():
-                if isinstance(val, (dict, list)):
-                    traverse_yaml(val)
-        elif isinstance(obj, list):
-            for e in obj:
-                if isinstance(e, (dict, list)):
-                    traverse_yaml(e)
-        else:
-            return
+def get_yaml_files(options: Namespace) -> dict:
+    """Find all yaml files."""
+    # git is preferred as it also considers .gitignore
+    git_command = ['git', 'ls-files', '*.yaml', '*.yml']
+    _logger.info("Discovering files to lint: %s", ' '.join(git_command))
 
-    task_comment_obj_strs = []
-    traverse_yaml(task)
+    out = None
 
-    rule_id_list = []
-    for comment_obj_str in task_comment_obj_strs:
-        for line in comment_obj_str.split('\\n'):
-            rule_id_list.extend(get_rule_skips_from_line(line))
+    try:
+        out = subprocess.check_output(
+            git_command,
+            stderr=subprocess.STDOUT,
+            universal_newlines=True
+        ).split()
+    except subprocess.CalledProcessError as exc:
+        _logger.warning(
+            "Failed to discover yaml files to lint using git: %s",
+            exc.output.rstrip('\n')
+        )
+    except FileNotFoundError as exc:
+        if options.verbosity:
+            _logger.warning(
+                "Failed to locate command: %s", exc
+            )
 
-    return rule_id_list
+    if out is None:
+        out = [
+            os.path.join(root, name)
+            for root, dirs, files in os.walk('.')
+            for name in files
+            if name.endswith('.yaml') or name.endswith('.yml')
+        ]
+
+    return OrderedDict.fromkeys(sorted(out))
 
 
-def get_rule_skips_from_line(line):
-    rule_id_list = []
-    if '# noqa' in line:
-        noqa_text = line.split('# noqa')[1]
-        rule_id_list = noqa_text.split()
-    return rule_id_list
+# FIXME: drop noqa once this function is made simpler
+# Ref: https://github.com/ansible/ansible-lint/issues/744
+def get_playbooks_and_roles(options=None) -> List[str]:  # noqa: C901
+    """Find roles and playbooks."""
+    if options is None:
+        options = {}
+
+    files = get_yaml_files(options)
+
+    playbooks = []
+    role_dirs = []
+    role_internals = {
+        'defaults',
+        'files',
+        'handlers',
+        'meta',
+        'tasks',
+        'templates',
+        'vars',
+    }
+
+    # detect role in repository root:
+    if 'tasks/main.yml' in files or 'tasks/main.yaml' in files:
+        role_dirs.append('.')
+
+    for p in map(Path, files):
+
+        try:
+            for file_path in options.exclude_paths:
+                if str(p.resolve()).startswith(str(file_path)):
+                    raise FileNotFoundError(
+                        f'File {file_path} matched exclusion entry: {p}')
+        except FileNotFoundError as e:
+            _logger.debug('Ignored %s due to: %s', p, e)
+            continue
+
+        if (next((i for i in p.parts if i.endswith('playbooks')), None) or
+                'playbook' in p.parts[-1]):
+            playbooks.append(normpath(p))
+            continue
+
+        # ignore if any folder ends with _vars
+        if next((i for i in p.parts if i.endswith('_vars')), None):
+            continue
+        elif 'roles' in p.parts or '.' in role_dirs:
+            if 'tasks' in p.parts and p.parts[-1] in ['main.yaml', 'main.yml']:
+                role_dirs.append(str(p.parents[1]))
+            elif role_internals.intersection(p.parts):
+                continue
+            elif 'tests' in p.parts:
+                playbooks.append(normpath(p))
+        if 'molecule' in p.parts:
+            if p.parts[-1] != 'molecule.yml':
+                playbooks.append(normpath(p))
+            continue
+        # hidden files are clearly not playbooks, likely config files.
+        if p.parts[-1].startswith('.'):
+            continue
+
+        if is_playbook(str(p)):
+            playbooks.append(normpath(p))
+            continue
+
+        _logger.info('Unknown file type: %s', normpath(p))
+
+    _logger.info('Found roles: %s', ' '.join(role_dirs))
+    _logger.info('Found playbooks: %s', ' '.join(playbooks))
+
+    return role_dirs + playbooks
+
+
+def expand_path_vars(path: str) -> str:
+    """Expand the environment or ~ variables in a path string."""
+    # It may be possible for function to be called with a Path object
+    path = str(path).strip()
+    path = os.path.expanduser(path)
+    path = os.path.expandvars(path)
+    return path
+
+
+def expand_paths_vars(paths: List[str]) -> List[str]:
+    """Expand the environment or ~ variables in a list."""
+    paths = [expand_path_vars(p) for p in paths]
+    return paths
+
+
+def get_rules_dirs(rulesdir: List[str], use_default: bool) -> List[str]:
+    """Return a list of rules dirs."""
+    default_ruledirs = [DEFAULT_RULESDIR]
+    default_custom_rulesdir = os.environ.get(
+        CUSTOM_RULESDIR_ENVVAR, os.path.join(DEFAULT_RULESDIR, "custom")
+    )
+    custom_ruledirs = sorted(
+        str(rdir.resolve())
+        for rdir in Path(default_custom_rulesdir).iterdir()
+        if rdir.is_dir() and (rdir / "__init__.py").exists()
+    )
+    if use_default:
+        return rulesdir + custom_ruledirs + default_ruledirs
+
+    return rulesdir or custom_ruledirs + default_ruledirs
