@@ -1,9 +1,11 @@
 import unittest
 
-from ansiblelint import RulesCollection
-from ansiblelint.rules.UseHandlerRatherThanWhenChangedRule import UseHandlerRatherThanWhenChangedRule
-from test import RunFromText
+from ansiblelint.rules import RulesCollection
+from ansiblelint.rules.UseHandlerRatherThanWhenChangedRule import (
+    UseHandlerRatherThanWhenChangedRule,
+)
 
+from . import RunFromText
 
 SUCCESS_TASKS = '''
 - name: print helpful error message
